from django.http import JsonResponse

from nwon_django_toolbox.exception_handler.get_response import get_response


class ExceptionMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):

        response = self.get_response(request)

        if response.status_code == 500:
            response = get_response(
                message="Internal server error, please try again later",
                status_code=response.status_code,
            )
            return JsonResponse(response, status=response["status_code"])

        if response.status_code == 404 and "Page not found" in str(response.content):
            response = get_response(
                message="Page not found, invalid url", status_code=response.status_code
            )
            return JsonResponse(response, status=response["status_code"])

        return response



from typing import Type, Union

from django.db.models import Model
from django.urls import reverse
from django.utils.html import format_html

from nwon_django_toolbox.typings.model_type import ModelType


def detail_url_for_model(
    model: ModelType[Model], primary_key: Union[str, int, None] = None
):
    primary_key = primary_key if primary_key else model.id
    return detail_url_for_model_class(model.__class__, primary_key)


def detail_url_for_model_class(model: Type[Model], primary_key: Union[str, int]):
    view_name = _base_name_from_model_class(model) + "-detail"

    url = reverse(
        viewname=view_name,
        kwargs={"pk": primary_key},
    )
    return url


def list_url_for_model_class(model_class: Type[Model]):
    return reverse(_base_name_from_model_class(model_class) + "-list")


def list_url_for_model(model: ModelType[Model]):
    return list_url_for_model_class(model.__class__)


def formatted_django_admin_link(
    model: ModelType[Model], application: str = "tamoc"
) -> str:
    url = django_admin_link_for_model(model, application)
    return format_html("<a href='{url}'>{text}</a>", url=url, text=model.__str__())


def django_admin_link_for_model(
    model: ModelType[Model], application: str = "tamoc"
) -> str:
    return reverse(
        "admin:%s_%s_change" % (application, _base_name_from_model(model)),
        args=(model.pk,),
    )


def _base_name_from_model_class(model_class: Type[Model]) -> str:
    return model_class.__name__.lower()


def _base_name_from_model(model: ModelType[Model]) -> str:
    return _base_name_from_model_class(model.__class__)


__all__ = [
    "detail_url_for_model",
    "detail_url_for_model_class",
    "list_url_for_model_class",
    "list_url_for_model",
    "formatted_django_admin_link",
    "django_admin_link_for_model",
]



from typing import Generic, Iterator, TypeVar, Union

from django.db.models import QuerySet

T = TypeVar("T")


class ModelType(Generic[T]):
    def __iter__(self) -> Iterator[Union[T, QuerySet]]:
        pass


__all__ = ["ModelType"]



from nwon_django_toolbox.fields.encrypted_text_field import EncryptedTextField
from nwon_django_toolbox.fields.pydantic_json_field import (
    ModelSerializerWithPydantic,
    PydanticJsonField,
    PydanticJsonFieldSerializer,
)

__all__ = [
    "EncryptedTextField",
    "PydanticJsonFieldSerializer",
    "PydanticJsonField",
    "ModelSerializerWithPydantic",
]



from typing import List, Union


def get_response(
    message: str = "",
    result: Union[dict, List, None] = None,
    status_code: int = 200,
):
    return {
        "message": message,
        "result": result,
        "status_code": status_code,
    }



from rest_framework.views import exception_handler

from nwon_django_toolbox.exception_handler.get_error_message import get_error_message
from nwon_django_toolbox.exception_handler.get_response import get_response


def handle_exception(exc, context):
    error_response = exception_handler(exc, context)

    if error_response is not None:
        error = error_response.data

        if isinstance(error, list) and error:
            if isinstance(error[0], dict):
                error_response.data = get_response(
                    message=get_error_message(error[0]),
                    status_code=error_response.status_code,
                    result=error,
                )

            elif isinstance(error[0], str):
                error_response.data = get_response(
                    message=error[0],
                    status_code=error_response.status_code,
                    result=error,
                )

        if isinstance(error, dict):
            error_response.data = get_response(
                message=get_error_message(error),
                status_code=error_response.status_code,
                result=error,
            )
    return error_response



from nwon_django_toolbox.exception_handler.exception_handler import exception_handler
from nwon_django_toolbox.exception_handler.exception_middleware import (
    ExceptionMiddleware,
)

__all__ = ["exception_handler", "ExceptionMiddleware"]



def get_error_message(error_dict: dict):
    field = next(iter(error_dict))
    response = error_dict[next(iter(error_dict))]

    if isinstance(response, dict):
        response = get_error_message(response)
    elif isinstance(response, list):
        response_message = response[0]
        if isinstance(response_message, dict):
            response = get_error_message(response_message)
        else:
            response = response[0]
    return response
